@@ -1,4 +1,4 @@
-make_check_measures = function(task_type) {
+make_check_measures = function(task_type = NULL) {
   crate(function(x) {
     if (is.null(x)) {
       return(TRUE)
@@ -17,9 +17,11 @@ make_check_measures = function(task_type) {
     if ("epoch" %in% ids(x)) {
       stopf("Measure must not have id 'epoch'.")
     }
-    # some measures have task_type NA, which means they work with all task types
-    if (!all(map_lgl(map(x, "task_type"), function(x) task_type %in% x || (length(x) == 1L && is.na(x))))) {
-      return(sprintf("Measures must support task type \"%s\".", task_type))
+    if (!is.null(task_type)) {
+      # some measures have task_type NA, which means they work with all task types
+      if (!all(map_lgl(map(x, "task_type"), function(x) task_type %in% x || (length(x) == 1L && is.na(x))))) {
+        return(sprintf("Measures must support task type \"%s\".", task_type))
+      }
     }
     f = function(x) "requires_learner" %in% x || "requires_model" %in% x
     if (any(map_lgl(map(x, "properties"), f))) {
@@ -32,6 +34,7 @@ make_check_measures = function(task_type) {
 
 check_measures_regr = make_check_measures("regr")
 check_measures_classif = make_check_measures("classif")
+check_measures = make_check_measures()
 
 epochs_aggr = function(x) as.integer(ceiling(mean(unlist(x))))
 
@@ -46,15 +49,17 @@ paramset_torchlearner = function(task_type) {
   check_measures = switch(task_type,
     regr = check_measures_regr,
     classif = check_measures_classif,
-    stopf("Unsupported task type \"%s\".", task_type)
+    make_check_measures(task_type)
   )
 
   param_set = ps(
     epochs                = p_int(tags = c("train", "validation", "internal_tuning", "required"), lower = 0L,
-      aggr = epochs_aggr, in_tune_fn = epochs_tune_fn, disable_in_tune = list(patience = 0)),
+      aggr = epochs_aggr, in_tune_fn = epochs_tune_fn, disable_in_tune = list(patience = 0, measures_valid = list())),
     device                = p_fct(tags = c("train", "predict", "required"), levels = mlr_reflections$torch$devices, init = "auto"),
     num_threads           = p_int(lower = 1L, tags = c("train", "predict", "required", "threads"), init = 1L),
+    num_interop_threads   = p_int(lower = 1L, tags = c("train", "predict", "required"), init = 1L),
     seed                  = p_int(tags = c("train", "predict", "required"), special_vals = list("random", NULL), init = "random"),
+    jit_trace             = p_lgl(init = FALSE, tags = c("train", "required")),
     # evaluation
     eval_freq             = p_int(lower = 1L, tags = c("train", "required"), init = 1L),
     measures_train        = p_uty(tags = c("train", "required"), custom_check = check_measures, init = list()),
@@ -64,7 +69,7 @@ paramset_torchlearner = function(task_type) {
     min_delta             = p_dbl(lower = 0, tags = c("train", "required"), init = 0),
     # dataloader parameters
     batch_size            = p_int(tags = c("train", "predict", "required"), lower = 1L),
-    shuffle               = p_lgl(tags = "train", default = FALSE),
+    shuffle               = p_lgl(tags = "train", default = FALSE, init = TRUE),
     sampler               = p_uty(tags = c("train", "predict")),
     batch_sampler         = p_uty(tags = c("train", "predict")),
     num_workers           = p_int(lower = 0, default = 0, tags = c("train", "predict")),
@@ -74,7 +79,9 @@ paramset_torchlearner = function(task_type) {
     timeout               = p_dbl(default = -1, tags = c("train", "predict")),
     worker_init_fn        = p_uty(tags = c("train", "predict")),
     worker_globals        = p_uty(tags = c("train", "predict")),
-    worker_packages       = p_uty(tags = c("train", "predict"), custom_check = check_character, special_vals = list(NULL))
+    worker_packages       = p_uty(tags = c("train", "predict"), custom_check = check_character, special_vals = list(NULL)),
+    tensor_dataset        = p_fct(levels = "device", init = FALSE, tags = c("train", "predict"), special_vals = list(FALSE, TRUE))
   )
   return(param_set)
 }
+
